const process = require('process')
const { exec } = require('child_process')
const util = require('util')
const { existsSync } = require('fs')
const chalk = require('chalk')
const ora = require('ora')
const boxen = require('boxen')
const {
  parseISO,
  format,
  addDays,
  addYears,
  isWeekend,
  setHours,
  setMinutes,
  isFuture
} = require('date-fns')

const execAsync = util.promisify(exec);

const makeHistory = function({ commitsPerDay, workdaysOnly, startDate, endDate, repositoryName }) {
  if (!commitsPerDay)
    commitsPerDay = '0,3'
  startDate = startDate ? parseISO(startDate) : addYears(new Date(), -1)
  endDate = endDate ? parseISO(endDate) : new Date()

  const commitDateList = createCommitDateList({
    workdaysOnly,
    commitsPerDay: commitsPerDay.split(","),
    startDate: startDate,
    endDate: endDate
  });

  (async function() {
    let spinner;
    
    spinner = ora("Creating your GitHub repository\n").start();
    
    const historyFolder = "my-history";

    // Remove git history folder in case if it already exists.]
    if (existsSync(`./${historyFolder}`)) {
      await execAsync(
        `${
          process.platform === "win32" ? "rmdir /s /q" : "rm -rf"
        } ${historyFolder}`
      );
    }

    // Create git history folder.
    await execAsync(`mkdir ${historyFolder}`);
    process.chdir(historyFolder);
    await execAsync(`git init`);

    // Create new repository
    startDate = format(startDate, "yyyy_MM_dd")
    endDate = format(endDate, "yyyy_MM_dd")
    repositoryName = repositoryName ? repositoryName : `history-${startDate}-${endDate}`
    const repository = await execAsync(`gh repo create ${repositoryName} -d "Activity history from ${startDate} to ${endDate}" --source=. --private`)
    repositoryName = repository.stdout.replace('https://github.com/', '').replace('\n', '');

    spinner.succeed();

    spinner = ora("Generating your GitHub activity\n").start();

    // Create commits.
    for (const date of commitDateList) {
      // Change spinner so user can get the progress right now.
      const dateFormatted = new Intl.DateTimeFormat("en", {
        day: "numeric",
        month: "long",
        year: "numeric"
      }).format(date);
      spinner.text = `Generating your Github activity... (${dateFormatted})\n`;

      await execAsync(`echo "${date}" > foo.txt`);
      await execAsync(`git add .`);
      await execAsync(`git commit --quiet --date "${date}" -m "fake commit"`);
    }

    spinner.succeed();

    spinner = ora("Pushing activities into your GitHub repository\n").start();

    await execAsync(`git remote set-url origin git@github.com:${repositoryName}`)
    await execAsync(`git push -u origin master`)

    spinner.succeed();

    console.log(
      boxen(
        `${chalk.green("Success")} ${
          commitDateList.length
        } commits have been created in ${repositoryName} repository.
      If you rely on this tool, please consider sending me ETH. I would appreciate it.
      My address is 0x135c823878da9F9d71e9Fc486fA2FE62cE9861a3`,
        { borderColor: "yellow", padding: 1, align: "center" }
      )
    );

    process.chdir('..')
    await execAsync(
      `${
        process.platform === "win32" ? "rmdir /s /q" : "rm -rf"
      } ${historyFolder}`
    );
  })();
};

function getRandomIntInclusive(min, max) {
  min = Math.ceil(min);
  max = Math.floor(max);
  return Math.floor(Math.random() * (max - min + 1)) + min;
}

function createCommitDateList({
  commitsPerDay,
  workdaysOnly,
  startDate,
  endDate
}) {
  const commitDateList = [];
  let currentDate = startDate;

  while (currentDate <= endDate) {
    if (workdaysOnly && isWeekend(currentDate)) {
      currentDate = addDays(currentDate, 1);
      continue;
    }
    for (let i = 0; i < getRandomIntInclusive(...commitsPerDay); i++) {
      const dateWithHours = setHours(currentDate, getRandomIntInclusive(9, 16));
      const commitDate = setMinutes(
        dateWithHours,
        getRandomIntInclusive(0, 59)
      );
      commitDateList.push(commitDate);
    }
    currentDate = addDays(currentDate, 1);
  }

  return commitDateList;
}

module.exports = makeHistory